<?php
/**
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 * @author     Michael Landis <finastbeans@yahoo.com>
 */

// must be run within Dokuwiki
if(!defined('DOKU_INC')) die();

if(!defined('DOKU_PLUGIN')) define('DOKU_PLUGIN',DOKU_INC.'lib/plugins/');
require_once(DOKU_PLUGIN.'action.php');

define('GUESTPASS_TOKEN_LENGTH', 10);

class action_plugin_guestpass extends DokuWiki_Action_Plugin {

  /**
   * return some info
   */
  function getInfo() {
	return array(	
      'author' => 'Michael Landis',
      'email'  => 'finastbeans@yahoo.com',
      'date'   => '2008-01-18',
      'name'   => 'Guestpass',
      'desc'   => 'Generates guestpasses for protected namespaces',
      'url'    => 'http://www.hintersphere.com/',
    );
  }

  /**
   * register the eventhandlers
   */
  function register(&$contr) {
    $this->_log("Registering ACTION_ACT_PREPROCESS...\n");
    $contr->register_hook('ACTION_ACT_PREPROCESS',
                          'BEFORE',
                          $this,
                          'handle_act_preprocess',
                           array());
                           
    $this->_log("Registering PARSER_CACHE_USE...\n");
    $contr->register_hook('PARSER_CACHE_USE',
                         'BEFORE',
                          $this,
                          'handle_cache_use',
                           array());             
                           
  }

    
  /**
   * Checks if 'newguestpass' was given as action, if so we
   * do handle the event our self and no further checking takes place
   */
  function handle_act_preprocess(&$event, $param) {
    
    $this->_log("Event Name:[".$event->name."]");
    $this->_log("Event Data:[".$event->data."]");

    // first handle guestpass actions
    if ($event->data == 'guestpass') {

        // guestpass plugin knows how to handle "guestpass", dw doesn't...
        $event->preventDefault();    

        // guestpass with no token means "new guestpass"
        if ($_REQUEST['token'] == null) {
            $this->_log("Handling new guestpass...\n");
            $event->data = $this->_handle_newGuestpass();
        } else { // otherwise process a guestpass
            $this->_log("Processing guestpass...\n");
            $event->data = $this->_handle_guestpass($_REQUEST['token']);
        }
        
    } else if ($event->data == 'login') {
        $this->_log("Processing login...\n");
        $this->_handle_login();
    } else if ($event->data == 'register') {
        $this->_log("Processing register...\n");
        $event->data = $this->_handle_register($event);
    }
  }


  function handle_cache_use(&$event, $param) {

    if ($event->data == 'guestpass' || isset($_SESSION['guestpass'])) {
        // if a new guestpass was issued, disable cache so {{guestpass}} is parsed
        // {{guestpass}} will now show link to the new guestpass
        $this->_log("Forcing refresh...");
        $event->preventDefault();
        $event->stopPropagation();
        $event->result = false;
    
    }
  }
  
  
  /**
   * Signout the guestpass user if they actually choose to login/register
   */
  function _handle_login() {
    
      global $conf;
      global $USERINFO;
      global $INFO, $ID;
      global $auth;
      
      // if you are already logged in as guestpass user, log em out...
      if ($_SERVER['REMOTE_USER'] == $this->getConf('guestpass_user')) {
        
          $this->_log("handle loginform...(logout guestpass traveler)");  

          if(isset($_SESSION[DOKU_COOKIE]['auth']['user']))
              unset($_SESSION[DOKU_COOKIE]['auth']['user']);
          if(isset($_SESSION[DOKU_COOKIE]['auth']['pass']))
              unset($_SESSION[DOKU_COOKIE]['auth']['pass']);
          if(isset($_SESSION[DOKU_COOKIE]['auth']['info']))
              unset($_SESSION[DOKU_COOKIE]['auth']['info']);
          if(isset($_SERVER['REMOTE_USER']))
              unset($_SERVER['REMOTE_USER']);
          $USERINFO=null; //FIXME
          setcookie(DOKU_COOKIE,'',time()-600000,DOKU_REL);
          
          return;
      }
  }
  
  
  /**
   * If there is a guestpass in the session, override the register method so that we can
   * use the groups specified in the guestpass_reg_group parameter
   */
  function _handle_register(&$event) {
    
    global $auth;
    global $USERINFO;
    global $conf;
      
    $this->_log("handling register action...");  
    
    // only adjust groups if we're traveling on guestpass...
    if (! isset($_SESSION['guestpass'])) {
      $this->_log("No guestpass in session");  
      return 'register';
    }
    
    $this->_log("handling registration for guestpass travelers...");  

    // perform registration
    if (!($_POST['save'] && register())){
      $event->preventDefault();
      // repaint the form
      return 'register';
    }

    // ok - success registering, now update the groups for this user...
    $guestpass_group = $this->getConf('guestpass_reg_group');
    $this->_log("adding group...".$group);  

    // array of groups per config
    $grps = array($conf['defaultgroup']);
    $grps[] = $guestpass_group;

    // we are changing the groups    
    $changes = array();
    $changes['grps'] = $grps;
    
    $this->_log("making changes for: ".$_POST['login']." ...".$changes);  
    if (!$auth->modifyUser($_POST['login'], $changes)) {
        $event->preventDefault();
        return 'register';
    }
    
    // success goes to login
    return 'login';
  }


  /**
   * Creates a new guestpass
   */
  function _handle_newGuestpass() {

    global $ID;
    global $INFO;
    global $TEXT;
    global $auth;
    
    $this->_log("new guestpass...\n");  

    // Make sure user is authorized to read original page
    if ($INFO['perm'] < AUTH_READ) {
        return 'denied';
    }

    // generate a new guestpass token
    $token = substr(md5(uniqid(rand(), true)), 0, GUESTPASS_TOKEN_LENGTH);
    $this->_log($token);
    
    // create a url for this guestpass
    $url = getBaseURL('true').$this->getConf('controller').'?do=guestpass&token='.$token;

    // save the original ID
    $originalID = $ID;
    
    // save the original title, use first header or ID...
    $metaTitle = $INFO['meta']['title'];
    $originalTitle = ($metaTitle ? $metaTitle : $ID);
        
    // where do we keep the guestpasses themselves?
    $namespace = $this->getConf('guestpass_namespace');
    $this->_log('namespace: '.$namespace);
        
    // create a guestpass as a page in a namespace
    $ID = $namespace.':'.$token;
    $INFO = pageinfo();
    
    $this->_log('INFO: '.$INFO);
        
    //check if locked by anyone - if not lock for myself      
    if ($INFO['locked']) {
        return 'locked';
    } else {
        lock($ID);
    }
    
    // get expiration date - 
    $days2live  = $this->getConf('days2live');
    $expireTimestamp = time() + ($days2live * 60 * 60 * 24);
    $this->_log("date_format[".$this->getConf('date_format')."]");
    $expireDate = date($this->getConf('date_format'), $expireTimestamp);
    
    // write guestpass page
    $TEXT = '====== '.$this->getLang('details_about').$originalTitle." ======\n\n";
    $TEXT .= $this->getLang('details_link')."**".$url."**\n\n";
    $TEXT .= $this->getLang('details_expires').$expireDate."\n\n----\n";
    $TEXT .= "[[".$originalID."|".$this->getLang('details_link_back')."]]\n\n";
    saveWikiText($ID,$TEXT,"Creating Guestpass",false);
    
    // store the original ID as the "source", per Dublin Core Metadata
    // TODO::Document this additional metadata
    p_set_metadata($ID, array('source' => $originalID, 'expires' => $expireTimestamp));
    
    unlock($ID);
    return 'show'; 
  }
    
    
  /**
   * Process a guestpass
   */
  function _handle_guestpass($token) {
    
    global $ID;
    global $INFO;
	global $USERINFO;   // available groups stored here...
	global $auth;


    // where do we keep the guestpasses themselves?
    $namespace = $this->getConf('guestpass_namespace');
    $this->_log('namespace: '.$namespace);
        
    // load a guestpass as a page in a namespace
    $ID = $namespace.':'.$token;
    $INFO = pageinfo();
    
    // get the destination page and expiration
    $sourceID = $INFO['meta']['source'];
    $expireTimestamp = $INFO['meta']['expires'];

    $this->_log('sourceID: '.$sourceID);
    $this->_log('expires: '.date($this->getConf('date_format'), $expireTimestamp));
        
    // check to see if guestpass is expired
    if (!$INFO['exists'] || time() > $expireTimestamp) {
        
        // delete the guestpass if it still exists
        if ($INFO['exists']) {
            $this->_deletepage();
        }

		// TODO::Create a guestpass expired page (get from conf)
        return 'denied';
    }
   
 	// log the user in with the guestpass user...
	// (unless user is logged in already)
	if (!$_SERVER['REMOTE_USER']) {
	   
	    // add guestpass to the session
	    session_start();
        $_SESSION['guestpass'] = $token;
        
        // log the user in...
        $user = $this->getConf('guestpass_user');
        $pass = $this->getConf('guestpass_pass');

        $_SERVER['REMOTE_USER'] = $user;
        $USERINFO = $auth->getUserData($user);
        
        // set cookie
        $pass   = PMA_blowfish_encrypt($pass,auth_cookiesalt());
        $cookie = base64_encode("$user|$sticky|$pass");
        if($sticky) $time = time()+60*60*24*365; // never invoked
        setcookie(DOKU_COOKIE,$cookie,$time,DOKU_REL);

        // set session
        $_SESSION[DOKU_COOKIE]['auth']['user'] = $user;
        $_SESSION[DOKU_COOKIE]['auth']['pass'] = $pass;
        $_SESSION[DOKU_COOKIE]['auth']['buid'] = auth_browseruid();
        $_SESSION[DOKU_COOKIE]['auth']['info'] = $USERINFO;
        $_SESSION[DOKU_COOKIE]['auth']['time'] = time();
	}
	
    // load the destination page
    $ID = $sourceID;
    $INFO = pageinfo();
    
    return 'show';    
  }
   
   
  // delete the current page indicated by $ID     
  function _deletepage() {

    global $ID;
    global $INFO;
      
    //check if locked by anyone - if not lock for myself      
    if ($INFO['locked']) {
        return 'locked';
    } else {
        lock($ID);
    }

    // delete page by removing all text and saving
    $TEXT = '';
    saveWikiText($ID,$TEXT,"Deleting Guestpass",false);

    unlock($ID);
  }
  
  
  function _log($msg) {
    $handle = fopen("./guestpass_error.log", "a");
    fwrite($handle, $msg."\n");
    fclose($handle);
  }

}

//Setup VIM: ex: et ts=4 enc=utf-8 :
