<?php
/**
 * Plugin Guestpass: Generates guestpass links and processes guestpass urls"
 *
 * Syntax: {{guestpass}}"
 * 
 * @license    GPL 2 (http://www.gnu.org/licenses/gpl.html)
 * @author     Michael Landis <finastbeans@yahoo.com>
 */
 
if (!defined('DOKU_INC')) die();

if (!defined('DOKU_LF')) define('DOKU_LF', "\n");
if (!defined('DOKU_TAB')) define('DOKU_TAB', "\t");
if (!defined('DOKU_PLUGIN')) define('DOKU_PLUGIN', DOKU_INC.'lib/plugins/'); 
 
require_once(DOKU_PLUGIN.'syntax.php');

/**
 * All DokuWiki plugins to extend the parser/rendering mechanism
 * need to inherit from this class
 */
class syntax_plugin_guestpass_guestpass extends DokuWiki_Syntax_Plugin {
 
    function getInfo(){
        return array(
            'author' => 'Michael Landis',
            'email'  => 'finastbeans@yahoo.com',
            'date'   => '2008-01-17',
            'name'   => 'Guestpass',
            'desc'   => 'Generate a guestpass link to the current page, otherwise protected by ACLs',
            'url'    => 'http://hintersphere.com',
        );
    }
 
    function getType(){
        return 'substition';
    }
 
    function getPType(){
        return 'block';
    }
 
   /**
    * TODO::Assign a real value
    */
    function getSort(){
        return 307;
    }
 
 
    function connectTo($mode) {
        $this->Lexer->addSpecialPattern('\{\{guestpass\}\}',$mode,'plugin_guestpass_guestpass');
    }

    function handle($match, $state, $pos, &$handler){ return array($match, $state, $pos); }
 
   /**
    * Generate a link to share this page (or view the guestpass you are traveling on).
    */
    function render($mode, &$renderer, $data) {
        
        if ($mode != 'xhtml') { 
            return false; 
        }
        
        $renderer->info['cache'] = false;
        $renderer->doc .= $this->_guestpassLink();
        
        return false;
    }
    
    
  /**
   * Displays a link used to generate a guestpass link to the current page
   * add do (action) and id 
   */
  function _guestpassLink() {

      global $lang;
      global $ID;
      global $USERINFO;
      
      // if traveling on guestpass, display a link to it
      if (isset($_SESSION['guestpass'])) {
          $link = '<a href="'.getBaseURL('true').$this->getConf('controller');
          $link .= '/'.$this->getConf('guestpass_namespace').'/'.$_SESSION['guestpass'].'" />'.$this->getLang('link_traveling').'</a>';
          return $link;     
      } 

      // ...otherwise, only display guestpass link for the guestpass group
      $groups = $USERINFO['grps'];
      if(!is_array($groups)) $groups = array();
      if (in_array($this->getConf('guestpass_reg_group'), $groups)) {
          $link = '<a href="'.getBaseURL('true').$this->getConf('controller');
          $link .= '?do=guestpass&id='.$ID.'" />'.$this->getLang('link_create').'</a>';
          return $link;
      }
      
      // otherwise paint nothing in the {{guestpass}} spot
      return '';
  }  

    
  function _log($msg) {
//    $handle = fopen("./guestpass_error.log", "a");
//    fwrite($handle, $msg."\n");
//    fclose($handle);
  }

}
 
